<?php

namespace Drupal\google_calendar\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\google_calendar\Entity\GoogleCalendarInterface;

use Drupal\google_calendar\GoogleCalendarImportEvents;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class GoogleCalendarImportEventsController.
 *
 * @package \Drupal\google_calendar
 */
class GoogleCalendarImportEventsController extends ControllerBase {

  /**
   * @var \Drupal\Core\Logger\LoggerChannel $logger
   *   The logger to use for status messages.
   */
  protected $logger;

  /**
   * @var \Drupal\google_calendar\GoogleCalendarImportEventsInterface
   *   The importer.
   */
  protected $googleCalendarImport;

  /**
   * Constructs a new GoogleCalendarImportEventsController object.
   *
   * @param \Drupal\google_calendar\GoogleCalendarImportEvents $google_calendar_import
   *   The importer.
   * @param \Drupal\Core\Logger\LoggerChannelFactory $loggerFactory
   *   Logger factory for messages.
   */
  public function __construct(GoogleCalendarImportEvents $google_calendar_import, LoggerChannelFactory $loggerFactory) {
    $this->googleCalendarImport = $google_calendar_import;
    $this->logger = $loggerFactory->get('google_calendar');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('google_calendar.sync_events'),
      $container->get('logger.factory')
    );
  }

  /**
   * Synchronize one Calendar.
   *
   * @param \Drupal\google_calendar\Entity\GoogleCalendarInterface $calendar
   *   The calendar entity being synchronized.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The next page.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Google_Service_Exception
   */
  public function synchronizeCalendar(GoogleCalendarInterface $calendar): RedirectResponse {

    $this->logger->info('Importing calendar @id: @name',
             [
               '@id' => $calendar->id(),
               '@name' => $calendar->getName(),
             ]);

    $this->googleCalendarImport->import($calendar);

    $this->logger->info('Events for calendar @id: @name Calendar have been imported successfully',
             [
               '@id' => $calendar->id(),
               '@name' => $calendar->getName(),
               '@result' => $calendar->getSyncResult(),
             ]);

    return $this->redirect('entity.google_calendar.collection');
  }

  /**
   * Synchronize all currently imported Calendars.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The next page.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Google_Service_Exception
   */
  public function synchronizeCalendars(): RedirectResponse {

    $foundCalendars = \Drupal::entityTypeManager()
      ->getStorage('google_calendar')
      ->loadByProperties(['status' => 1]);

    foreach ($foundCalendars as $calendar) {
      $this->logger->info('Importing calendar @id: @name',
               [
                 '@id' => $calendar->id(),
                 '@name' => $calendar->getName(),
               ]);

      $this->googleCalendarImport->import($calendar);

      $this->logger->info('Events for calendar @id: @name Calendar have been imported successfully',
               [
                 '@id' => $calendar->id(),
                 '@name' => $calendar->getName(),
                 '@result' => $calendar->getSyncResult(),
               ]);
    }

    return $this->redirect('entity.google_calendar.collection');
  }

}
