<?php

namespace Drupal\google_calendar;

use Drupal\google_calendar\Entity\GoogleCalendar;
use Drupal\google_calendar\Entity\GoogleCalendarInterface;

/**
 * Class GoogleCalendarImportEvents.
 */
interface GoogleCalendarImportEventsInterface {
  public const CLEANUP_NONE = 'no cleanup';
  public const CLEANUP_DEL_OLD = 'delete past events';
  public const CLEANUP_DEL_ALL = 'delete all events';
  public const CLEANUP_UNPUB_OLD = 'unpublish past events';
  public const CLEANUP_UNPUB_ALL = 'unpublish all events';

  public const OWNER_FIXED = 'fixed';
  public const OWNER_BYEMAIL = 'by_email';
  public const OWNER_BYNAME = 'by_name';

  /**
   * How far into the past or into the future should event changes be noted?
   *
   * Default of -1 hour allows for last minute changes to an event to be
   * recorded, should that be needed. 'now' and 'today' are also good options.
   *
   * Default of 1 year 1 day allows for convenience of booking annual events.
   */
  public function setEventHorizons(string $past, string $future): GoogleCalendarImportEventsInterface;

  /**
   * How far into the past or into the future should event changes be noted?
   *
   * @see setEventHorizonRefresh()
   */
  public function getEventHorizons();

  /**
   * How often should we recalculate when 'past' and 'future' are?
   *
   * "Recalculating" means refreshing all existing events, rather than relying
   * on an incremental update, and is hence slow.
   *
   * Setting 1 day ("P1D") is normal; reduce ("PT8H") when 'future' is not very far
   * away and hence changes are of greater importance; increase ("P7D") to reduce
   * bandwidth requirements at the expense of ignoring some changes.
   *
   * Values must be formatted as a DateInterval().
   */
  public function setEventHorizonRefresh(string $intv): GoogleCalendarImportEventsInterface;

  /**
   * How often should we recalculate when 'past' and 'future' are?
   *
   * @see setEventHorizonRefresh()
   */
  public function getEventHorizonRefresh();

  /**
   * String policy value determining if and how events are cleaned up during an
   * import.
   */
  public function setCleanupPolicy(string $policy): GoogleCalendarImportEventsInterface;

  /**
   * String policy value determining if and how events are cleaned up during an
   * import.
   */
  public function getCleanupPolicy();

  /**
   * How to determine who in Drupal terms is the owner of an event.
   *
   * One of the values in the OWNER_* constants:
   *   - fixed: the 'default owner' is always used.
   *   - by_name: the owner name from the event is looked up in the Drupal
   *     user table. If not found, the default owner is used.
   *   - by_email: the email addr from the event is looked up in the Drupal
   *     user table. If not found, the default owner is used.
   */
  public function setEventOwnership(string $policy): GoogleCalendarImportEventsInterface;

  /**
   * How to determine who in Drupal terms is the owner of an event.
   *
   * @see setEventOwnership()
   */
  public function getEventOwnership();

  /**
   * Nominated owner of events not having a otherwise-defined owner.
   * How this is used depends on the eventOwnership setting.
   *
   * string
   */
  public function setDefaultEventOwner(string $owner): GoogleCalendarImportEventsInterface;

  /**
   * Nominated owner of events not having a otherwise-defined owner.
   *
   * @see setDefaultEventOwner()
   */
  public function getDefaultEventOwner();

  /**
   * Reset event update statistics for a new import.
   */
  public function resetStats(): GoogleCalendarImportEventsInterface;

  /** Return number of events reported by Google. */
  public function getStatNewEvents(): int;

  /** Return number of events updated this time. */
  public function getStatModifyEvents(): int;

  /** Return number of events not seen before. */
  public function getStatCreatedEvents(): int;

  /** Return number of events actually saved. */
  public function getStatSavedEvents(): int;

  /** Return number of pages of events read from API. */
  public function getPageCount(): int;

  /**
   * Return a Sync Token, or NULL if no token should be used.
   *
   * A token will be returned unless either 'ignoresynctoken' is set
   * or the calendar resync schedule has been reached.
   *
   * @param $calendar
   *   The calendar for which the token is needed.
   * @param $ignoreSyncToken
   *   Flag forcing the function to return NULL.
   *
   * @return null|string
   */
  function getSyncToken(GoogleCalendarInterface $calendar, bool $ignoreSyncToken);

  /**
   * Mark the Sync Token as invalid.
   *
   * @param GoogleCalendarInterface $entity
   *
   * @return int
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function deleteSyncToken(GoogleCalendarInterface $entity): int;

  /**
   * Import changes from Google Calendar to the google_event entities.
   *
   * @param \Drupal\google_calendar\Entity\GoogleCalendar $calendar
   * @param bool $ignoreSyncToken
   *
   * @return \Drupal\google_calendar\Entity\GoogleCalendar
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Google_Service_Exception
   */
  public function import(GoogleCalendar $calendar, $ignoreSyncToken = FALSE);
}
