<?php

namespace Drupal\google_calendar\WizardForm;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\google_secrets\GoogleSecretsStoreException;
use Drupal\google_secrets\GoogleSecretsStoreManager;

class ConfigurationWizard_6 extends ConfigurationWizardBase {

  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'google_calendar_configuration_wizard_6';
  }

  /**
   * {@inheritdoc}.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = parent::buildForm($form, $form_state);

    $googleSecretsStoreManager = \Drupal::service('plugin.manager.google_secrets');
    /** @var \Drupal\google_secrets\GoogleSecretsStoreInterface[] $store_types */
    $store_types = $googleSecretsStoreManager->getDefinitions();

    $secret_type = \Drupal::config('google_calendar.settings')->get('client_secret_type');
    if (!$secret_type) {
      $secret_type = 'static_file';
      $store_config = [];
    }
    /** @var \Drupal\google_secrets\GoogleSecretsStoreInterface $store */
    $store = $googleSecretsStoreManager->createInstance($secret_type, $store_config);

    $file_id = NULL;
    $file_content = NULL;

    /* *****************************************  */
    /* ******        Introduction         ****** */

    $form['intro'] = [
      '#type' => 'details',
      '#title' => $this->t('Step 6: Import credentials file'),
      '#open' => TRUE,
    ];

    $form['intro']['para_1'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . t('The credentials file just downloaded from Google must be made available to the website. There are two supported configuration methods; add others by extending or overriding the Drupal service "google_calendar.secrets_file":') . '</p>'
    ];

    $items = [];
    foreach ($store_types as $id => $def) {
      $items[] = [
        '#type' => 'markup',
        '#markup' => '<p>' . $def['title']->render() . ' (' . $id . '): ' . $def['description']->render() . '</p>',
      ];
    }
    $form['intro']['para_2'] = [
      '#theme' => 'item_list',
      '#list_type' => 'ul',
      '#items' => $items,
      '#attributes' => ['class' => 'secret-stores'],
    ];
    $form['intro']['para_3'] = [
      '#type' => 'markup',
      '#markup' =>
        '<p>' . t('The secrets_file service is currently set to: @type', ['@type' => $secret_type]) . '</p>' .
        '<p>' . t('To change this you will need to update the site service file and then rebuild the cache.') . '</p>'
    ];

    /* *****************************************  */
    /* ******        Store Config         ****** */

    $form['client_secret_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Secrets Storage Settings'),
      '#attributes' => ['class' => ['secrets_configured']]
    ];

    $subform = [];
    $form['client_secret_settings'][] = $store->buildForm($subform, $form_state);

    $file_content = '';
    try {
      $file_content = $store->get();
    }
    catch (GoogleSecretsStoreException $ex) {
    }

    /* *****************************************  */
    /* ******        Current Secrets      ****** */

    $form['client_secret_container'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Secrets Storage Status'),
      '#attributes' => ['class' => ['secrets_configured']]
    ];
    if (!empty($file_content) && isset($file_content['type'])) {
      $form['client_secret_container']['#attributes']['class'][] = 'success';
      $form['client_secret_container']['client_secret_info'] = [
        '#type' => 'markup',
        '#markup' => $this->t('The secrets have been configured for account: @account.', ['@account' => $file_content['client_email'],]),
      ];
    }
    else {
      $form['client_secret_container']['#attributes']['class'][] = 'failure';
      $form['client_secret_container']['client_secret_info'] = [
        '#type' => 'markup',
        '#markup' => $this->t('The secrets have not been configured.'),
      ];
    }

    $form['step_6'] = [
      '#type' => 'checkbox',
      '#required' => TRUE,
      '#title' => $this->t('Google Credentials installed'),
      '#default_value' => $this->store->get('step_6') ?: '',
      '#description' => $this->t('Check this when you have configured the account credentials.')
    ];

    $form['actions']['previous'] = [
      '#type' => 'link',
      '#title' => $this->t('Previous'),
      '#attributes' => [
        'class' => ['button'],
      ],
      '#weight' => 0,
      '#url' => Url::fromRoute('google_calendar.config_wizard_five'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('google_calendar.settings');

    $googleSecretsStoreManager = \Drupal::service('plugin.manager.google_secrets');
    $secret_type = $form_state->getValue('secret_type');

    // If have not changed store type, give plugin the old config, otherwise
    // leave it empty.
    $old_secret_type = $config->get('client_secret_type');
    $store_config = [];
    if ($old_secret_type === $secret_type) {
      $store_config = $config->get('client_secret_config');
    }

    if (!$secret_type) {
      $secret_type = 'static_file';
    }
    /** @var \Drupal\google_secrets\GoogleSecretsStoreInterface $store */
    $store = $googleSecretsStoreManager->createInstance($secret_type, $store_config);
    $store_config = $store->submitForm($form, $form_state);

    $this->store->set('client_secret_type', $secret_type);
    $this->store->set('client_secret_config', $store_config);
    $this->store->set('step_6', $form_state->getValue('step_6'));

    // Save the data
    $this->saveData();
    $form_state->setRedirect('entity.google_calendar.collection');

  }
}
